@@ -183,12 +183,12 @@ class _TransactionFactory(object):
 
         The configurational options given here act as **defaults**
         when the :class:`._TransactionFactory` is configured using
-        a :class:`.oslo.config.cfg.ConfigOpts` object; the options
-        present within the :class:`.oslo.config.cfg.ConfigOpts` **take
+        a :class:`oslo_config.cfg.ConfigOpts` object; the options
+        present within the :class:`oslo_config.cfg.ConfigOpts` **take
         precedence** versus the arguments passed here.  By default,
         the :class:`._TransactionFactory` loads in the configuration from
-        :data:`oslo.config.cfg.CONF`, after applying the
-        :data:`oslo.db.options.database_opts` configurational defaults to it.
+        :data:`oslo_config.cfg.CONF`, after applying the
+        :data:`oslo_db.options.database_opts` configurational defaults to it.
 
         .. seealso::
 
@@ -206,7 +206,7 @@ class _TransactionFactory(object):
         Behavior here is the same as that of
         :meth:`._TransactionFactory.configure_defaults`,
         with the exception that values specified here will **supersede** those
-        setup in the :class:`.oslo.config.cfg.ConfigOpts` options.
+        setup in the :class:`oslo_config.cfg.ConfigOpts` options.
 
         .. seealso::
 
@@ -528,13 +528,13 @@ class _TransactionContext(object):
         else:
             transaction.rollback()
 
-    def _produce_block(self, mode, connection, savepoint):
+    def _produce_block(self, mode, connection, savepoint, allow_async=False):
         if mode is _WRITER:
             self._writer()
         elif mode is _ASYNC_READER:
             self._async_reader()
         else:
-            self._reader()
+            self._reader(allow_async)
         if connection:
             return self._connection(savepoint)
         else:
@@ -552,10 +552,10 @@ class _TransactionContext(object):
                 "Can't upgrade an ASYNC_READER transaction "
                 "to a WRITER mid-transaction")
 
-    def _reader(self):
+    def _reader(self, allow_async=False):
         if self.mode is None:
             self.mode = _READER
-        elif self.mode is _ASYNC_READER:
+        elif self.mode is _ASYNC_READER and not allow_async:
             raise TypeError(
                 "Can't upgrade an ASYNC_READER transaction "
                 "to a READER mid-transaction")
@@ -569,6 +569,9 @@ class _TransactionContextTLocal(threading.local):
     def __deepcopy__(self, memo):
         return self
 
+    def __reduce__(self):
+        return _TransactionContextTLocal, ()
+
 
 class _TransactionContextManager(object):
     """Provide context-management and decorator patterns for transactions.
@@ -586,7 +589,8 @@ class _TransactionContextManager(object):
             savepoint=False,
             connection=False,
             replace_global_factory=None,
-            _is_global_manager=False):
+            _is_global_manager=False,
+            allow_async=False):
 
         if root is None:
             self._root = self
@@ -603,6 +607,7 @@ class _TransactionContextManager(object):
             raise TypeError(
                 "setting savepoint and independent makes no sense.")
         self._connection = connection
+        self._allow_async = allow_async
 
     @property
     def _factory(self):
@@ -619,6 +624,17 @@ class _TransactionContextManager(object):
         """
         self._factory.configure(**kw)
 
+    def get_legacy_facade(self):
+        """Return a :class:`.LegacyEngineFacade` for factory from this context.
+
+        This facade will make use of the same engine and sessionmaker
+        as this factory, however will not share the same transaction context;
+        the legacy facade continues to work the old way of returning
+        a new Session each time get_session() is called.
+        """
+
+        return self._factory.get_legacy_facade()
+
     @property
     def replace(self):
         """Modifier to replace the global transaction factory with this one."""
@@ -634,6 +650,27 @@ class _TransactionContextManager(object):
         """Modifier to set the transaction to READER."""
         return self._clone(mode=_READER)
 
+    @property
+    def allow_async(self):
+        """Modifier to allow async operations
+
+        Allows async operations if asynchronous session is already
+        started in this context. Marking DB API methods with READER would make
+        it impossible to use them in ASYNC_READER transactions, and marking
+        them with ASYNC_READER would require a modification of all the places
+        these DB API methods are called to force READER mode, where the latest
+        DB state is required.
+
+        In Nova DB API methods should have a 'safe' default (i.e. READER),
+        so that they can start sessions on their own, but it would also be
+        useful for them to be able to participate in an existing ASYNC_READER
+        session, if one was started up the stack.
+        """
+
+        if self._mode is _WRITER:
+            raise TypeError("Setting async on a WRITER makes no sense")
+        return self._clone(allow_async=True)
+
     @property
     def independent(self):
         """Modifier to start a transaction independent from any enclosing."""
@@ -718,7 +755,8 @@ class _TransactionContextManager(object):
                 with current._produce_block(
                     mode=self._mode,
                     connection=self._connection,
-                    savepoint=self._savepoint) as resource:
+                    savepoint=self._savepoint,
+                    allow_async=self._allow_async) as resource:
                     yield resource
             else:
                 yield
@@ -824,7 +862,7 @@ def get_legacy_facade():
     a new Session each time get_session() is called.
 
     """
-    return _context_manager._factory.get_legacy_facade()
+    return _context_manager.get_legacy_facade()
 
 
 reader = _context_manager.reader
@@ -838,10 +876,8 @@ writer = _context_manager.writer
 class LegacyEngineFacade(object):
     """A helper class for removing of global engine instances from oslo.db.
 
-    .. deprecated::
-
-        EngineFacade is deprecated.  Please use
-        oslo.db.sqlalchemy.enginefacade for new development.
+    .. deprecated:: 1.12.0
+        Please use :mod:`oslo_db.sqlalchemy.enginefacade` for new development.
 
     As a library, oslo.db can't decide where to store/when to create engine
     and sessionmaker instances, so this must be left for a target application.
@@ -864,64 +900,62 @@ class LegacyEngineFacade(object):
        transactional context (i.e. it's not thread-safe). sessionmaker is
        a factory of sessions.
 
+    :param sql_connection: the connection string for the database to use
+    :type sql_connection: string
+
+    :param slave_connection: the connection string for the 'slave' database
+                             to use. If not provided, the master database
+                             will be used for all operations. Note: this
+                             is meant to be used for offloading of read
+                             operations to asynchronously replicated slaves
+                             to reduce the load on the master database.
+    :type slave_connection: string
+
+    :param sqlite_fk: enable foreign keys in SQLite
+    :type sqlite_fk: bool
+
+    :param autocommit: use autocommit mode for created Session instances
+    :type autocommit: bool
+
+    :param expire_on_commit: expire session objects on commit
+    :type expire_on_commit: bool
+
+    Keyword arguments:
+
+    :keyword mysql_sql_mode: the SQL mode to be used for MySQL sessions.
+                             (defaults to TRADITIONAL)
+    :keyword idle_timeout: timeout before idle sql connections are reaped
+                           (defaults to 3600)
+    :keyword connection_debug: verbosity of SQL debugging information.
+                               -1=Off, 0=None, 100=Everything (defaults
+                               to 0)
+    :keyword max_pool_size: maximum number of SQL connections to keep open
+                            in a pool (defaults to SQLAlchemy settings)
+    :keyword max_overflow: if set, use this value for max_overflow with
+                           sqlalchemy (defaults to SQLAlchemy settings)
+    :keyword pool_timeout: if set, use this value for pool_timeout with
+                           sqlalchemy (defaults to SQLAlchemy settings)
+    :keyword sqlite_synchronous: if True, SQLite uses synchronous mode
+                                 (defaults to True)
+    :keyword connection_trace: add python stack traces to SQL as comment
+                               strings (defaults to False)
+    :keyword max_retries: maximum db connection retries during startup.
+                          (setting -1 implies an infinite retry count)
+                          (defaults to 10)
+    :keyword retry_interval: interval between retries of opening a sql
+                             connection (defaults to 10)
+    :keyword thread_checkin: boolean that indicates that between each
+                             engine checkin event a sleep(0) will occur to
+                             allow other greenthreads to run (defaults to
+                             True)
+
     """
     def __init__(self, sql_connection, slave_connection=None,
                  sqlite_fk=False, autocommit=True,
                  expire_on_commit=False, _conf=None, _factory=None, **kwargs):
-        """Initialize engine and sessionmaker instances.
-
-        :param sql_connection: the connection string for the database to use
-        :type sql_connection: string
-
-        :param slave_connection: the connection string for the 'slave' database
-                                 to use. If not provided, the master database
-                                 will be used for all operations. Note: this
-                                 is meant to be used for offloading of read
-                                 operations to asynchronously replicated slaves
-                                 to reduce the load on the master database.
-        :type slave_connection: string
-
-        :param sqlite_fk: enable foreign keys in SQLite
-        :type sqlite_fk: bool
-
-        :param autocommit: use autocommit mode for created Session instances
-        :type autocommit: bool
-
-        :param expire_on_commit: expire session objects on commit
-        :type expire_on_commit: bool
-
-        Keyword arguments:
-
-        :keyword mysql_sql_mode: the SQL mode to be used for MySQL sessions.
-                                 (defaults to TRADITIONAL)
-        :keyword idle_timeout: timeout before idle sql connections are reaped
-                               (defaults to 3600)
-        :keyword connection_debug: verbosity of SQL debugging information.
-                                   -1=Off, 0=None, 100=Everything (defaults
-                                   to 0)
-        :keyword max_pool_size: maximum number of SQL connections to keep open
-                                in a pool (defaults to SQLAlchemy settings)
-        :keyword max_overflow: if set, use this value for max_overflow with
-                               sqlalchemy (defaults to SQLAlchemy settings)
-        :keyword pool_timeout: if set, use this value for pool_timeout with
-                               sqlalchemy (defaults to SQLAlchemy settings)
-        :keyword sqlite_synchronous: if True, SQLite uses synchronous mode
-                                     (defaults to True)
-        :keyword connection_trace: add python stack traces to SQL as comment
-                                   strings (defaults to False)
-        :keyword max_retries: maximum db connection retries during startup.
-                              (setting -1 implies an infinite retry count)
-                              (defaults to 10)
-        :keyword retry_interval: interval between retries of opening a sql
-                                 connection (defaults to 10)
-        :keyword thread_checkin: boolean that indicates that between each
-                                 engine checkin event a sleep(0) will occur to
-                                 allow other greenthreads to run (defaults to
-                                 True)
-        """
         warnings.warn(
             "EngineFacade is deprecated; please use "
-            "oslo.db.sqlalchemy.enginefacade",
+            "oslo_db.sqlalchemy.enginefacade",
             exception.OsloDBDeprecationWarning,
             stacklevel=2)
 
@@ -942,6 +976,10 @@ class LegacyEngineFacade(object):
                 _conf, connection=sql_connection,
                 slave_connection=slave_connection)
 
+    def _check_factory_started(self):
+        if not self._factory._started:
+            self._factory._start()
+
     def get_engine(self, use_slave=False):
         """Get the engine instance (note, that it's shared).
 
@@ -952,6 +990,7 @@ class LegacyEngineFacade(object):
         :type use_slave: bool
 
         """
+        self._check_factory_started()
         if use_slave:
             return self._factory._reader_engine
         else:
@@ -972,6 +1011,7 @@ class LegacyEngineFacade(object):
         was created). See SQLAlchemy Session docs for details.
 
         """
+        self._check_factory_started()
         if use_slave:
             return self._factory._reader_maker(**kwargs)
         else:
@@ -984,6 +1024,7 @@ class LegacyEngineFacade(object):
         be temporarily injected with some state such as a specific connection.
 
         """
+        self._check_factory_started()
         if use_slave:
             return self._factory._reader_maker
         else:
@@ -995,7 +1036,7 @@ class LegacyEngineFacade(object):
         """Initialize EngineFacade using oslo.config config instance options.
 
         :param conf: oslo.config config instance
-        :type conf: oslo.config.cfg.ConfigOpts
+        :type conf: oslo_config.cfg.ConfigOpts
 
         :param sqlite_fk: enable foreign keys in SQLite
         :type sqlite_fk: bool
