@@ -13,6 +13,7 @@
 import collections
 import contextlib
 import copy
+import pickle
 import warnings
 
 import mock
@@ -582,6 +583,60 @@ class MockFacadeTest(oslo_test_base.BaseTestCase):
             exc.args[0]
         )
 
+    def test_reader_allow_async_nested_in_async_reader(self):
+        context = oslo_context.RequestContext()
+
+        @enginefacade.reader.async
+        def go1(context):
+            context.session.execute("test1")
+            go2(context)
+
+        @enginefacade.reader.allow_async
+        def go2(context):
+            context.session.execute("test2")
+
+        go1(context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_async_reader_session(makers) as session:
+                    session.execute("test1")
+                    session.execute("test2")
+
+    def test_reader_allow_async_nested_in_reader(self):
+        context = oslo_context.RequestContext()
+
+        @enginefacade.reader.reader
+        def go1(context):
+            context.session.execute("test1")
+            go2(context)
+
+        @enginefacade.reader.allow_async
+        def go2(context):
+            context.session.execute("test2")
+
+        go1(context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_reader_session(makers) as session:
+                    session.execute("test1")
+                    session.execute("test2")
+
+    def test_reader_allow_async_is_reader_by_default(self):
+        context = oslo_context.RequestContext()
+
+        @enginefacade.reader.allow_async
+        def go1(context):
+            context.session.execute("test1")
+
+        go1(context)
+
+        with self._assert_engines() as engines:
+            with self._assert_makers(engines) as makers:
+                with self._assert_reader_session(makers) as session:
+                    session.execute("test1")
+
     def test_writer_nested_in_async_reader_raises(self):
         context = oslo_context.RequestContext()
 
@@ -984,9 +1039,46 @@ class LegacyIntegrationtest(test_base.DbTestCase):
             enginefacade._context_manager._factory._writer_maker
         )
 
+    def test_legacy_facades_from_different_context_managers(self):
+        transaction_context1 = enginefacade.transaction_context()
+        transaction_context2 = enginefacade.transaction_context()
+
+        transaction_context1.configure(connection='sqlite:///?conn1')
+        transaction_context2.configure(connection='sqlite:///?conn2')
+
+        legacy1 = transaction_context1.get_legacy_facade()
+        legacy2 = transaction_context2.get_legacy_facade()
+
+        self.assertNotEqual(legacy1, legacy2)
+
+    def test_legacy_not_started(self):
+
+        factory = enginefacade._TransactionFactory()
+
+        self.assertRaises(
+            exception.CantStartEngineError,
+            factory.get_legacy_facade
+        )
+
+        legacy_facade = factory.get_legacy_facade()
+        self.assertRaises(
+            exception.CantStartEngineError,
+            legacy_facade.get_session
+        )
+
+        self.assertRaises(
+            exception.CantStartEngineError,
+            legacy_facade.get_session
+        )
+
+        self.assertRaises(
+            exception.CantStartEngineError,
+            legacy_facade.get_engine
+        )
+
 
 class ThreadingTest(test_base.DbTestCase):
-    """Test copying on new threads using real connections and sessions."""
+    """Test copy/pickle on new threads using real connections and sessions."""
 
     def _assert_ctx_connection(self, context, connection):
         self.assertIs(context.connection, connection)
@@ -1162,6 +1254,21 @@ class ThreadingTest(test_base.DbTestCase):
         with self._patch_thread_ident():
             go_one(context)
 
+    def test_contexts_picklable(self):
+        context = oslo_context.RequestContext()
+
+        with enginefacade.writer.using(context) as session:
+            self._assert_ctx_session(context, session)
+
+            pickled = pickle.dumps(context)
+
+            unpickled = pickle.loads(pickled)
+
+            with enginefacade.writer.using(unpickled) as session2:
+                self._assert_ctx_session(unpickled, session2)
+
+                assert session is not session2
+
 
 class LiveFacadeTest(test_base.DbTestCase):
     """test using live SQL with test-provisioned databases.
