@@ -57,9 +57,14 @@ class _SQLAExceptionMatcher(object):
         else:
             self.assertEqual(str(exc.orig).lower(), message.lower())
         if sql is not None:
-            self.assertEqual(exc.statement, sql)
-        if params is not None:
-            self.assertEqual(exc.params, params)
+            if params is not None:
+                if '?' in exc.statement:
+                    self.assertEqual(exc.statement, sql)
+                    self.assertEqual(exc.params, params)
+                else:
+                    self.assertEqual(exc.statement % exc.params, sql % params)
+            else:
+                self.assertEqual(exc.statement, sql)
 
 
 class TestsExceptionFilter(_SQLAExceptionMatcher, oslo_test_base.BaseTestCase):
@@ -72,6 +77,9 @@ class TestsExceptionFilter(_SQLAExceptionMatcher, oslo_test_base.BaseTestCase):
 
         """
 
+    class DataError(Error):
+        pass
+
     class OperationalError(Error):
         pass
 
@@ -689,6 +697,28 @@ class TestDeadlock(TestsExceptionFilter):
         )
 
 
+class TestDataError(TestsExceptionFilter):
+    def _run_bad_data_test(self, dialect_name, message, error_class):
+        self._run_test(dialect_name,
+                       "INSERT INTO TABLE some_values",
+                       error_class(message),
+                       exception.DBDataError)
+
+    def test_bad_data_incorrect_string(self):
+        # Error sourced from https://bugs.launchpad.net/cinder/+bug/1393871
+        self._run_bad_data_test("mysql",
+                                '(1366, "Incorrect string value: \'\\xF0\' '
+                                'for column \'resource\' at row 1"',
+                                self.OperationalError)
+
+    def test_bad_data_out_of_range(self):
+        # Error sourced from https://bugs.launchpad.net/cinder/+bug/1463379
+        self._run_bad_data_test("mysql",
+                                '(1264, "Out of range value for column '
+                                '\'resource\' at row 1"',
+                                self.DataError)
+
+
 class IntegrationTest(test_base.DbTestCase):
     """Test an actual error-raising round trips against the database."""
 
