@@ -148,6 +148,23 @@ class TestPaginateQuery(test_base.BaseTestCase):
                              ['user_id', 'project_id'],
                              sort_dirs=['asc', 'desc'])
 
+    def test_invalid_sort_key_str(self):
+        self.assertEqual("Sort key supplied is invalid: None",
+                         str(exception.InvalidSortKey()))
+        self.assertEqual("Sort key supplied is invalid: lol",
+                         str(exception.InvalidSortKey("lol")))
+        self.assertEqual("Sort key supplied is invalid: lol",
+                         exception.InvalidSortKey("lol").message)
+
+    def test_invalid_unicode_paramater_str(self):
+        self.assertEqual(
+            "Invalid Parameter: Encoding directive wasn't provided.",
+            str(exception.DBInvalidUnicodeParameter()))
+        self.assertEqual(
+            "Invalid Parameter: Encoding directive wasn't provided.",
+            exception.DBInvalidUnicodeParameter().message
+        )
+
     def test_paginate_query_attribute_error(self):
         sqlalchemy.asc(self.model.user_id).AndReturn('asc')
         self.query.order_by('asc').AndReturn(self.query)
